//
//  MenuViewController.m
//  scummvm
//
//  Created by Ilya Levin on 10/10/2015.
//
//

#import "MenuViewController.h"
#import "SettingsViewController.h"
#import "UIFont+ScummVMAdditions.h"
#import "Strings.h"
#import "NSStringAdditions.h"
#import "UIColor+MTAdditions.h"
#import "UIButton+MTAdditions.h"
#import "NSArray+CPPConvertionAdditions.h"
#import "ExtraItemsListViewController.h"
#import "UIActivityViewController+MTAdditions.h"

#import <StoreKit/StoreKit.h>

#ifdef USE_GA

#import "GAI.h"
#import "GAIDictionaryBuilder.h"
#import "GAIFields.h"

#endif

using namespace Enhanced;

@interface MTStoreProductViewController : SKStoreProductViewController


@end

@implementation MTStoreProductViewController

-(BOOL)shouldAutorotate
{
    return NO;
}

@end

@interface MenuViewController () <SKStoreProductViewControllerDelegate>
@property (retain, nonatomic) IBOutlet UIButton *btnNewGame;
@property (retain, nonatomic) IBOutlet UIButton *btnSave;
@property (retain, nonatomic) IBOutlet UIButton *btnLoad;
@property (retain, nonatomic) IBOutlet UIButton *btnContinue;
@property (retain, nonatomic) IBOutlet UIButton *btnSettings;
@property (retain, nonatomic) IBOutlet UIButton *btnTutorial;

@property (retain, nonatomic) IBOutlet UIButton *btnExtras;
@property (retain, nonatomic) IBOutlet UIButton *btnQuit;
@property (retain, nonatomic) IBOutlet UIButton *btnAd;
@property (retain, nonatomic) IBOutlet UIButton *btnShare;
@property (retain, nonatomic) IBOutlet UIImageView *imgvAd;
@property (retain, nonatomic) IBOutlet UIImageView *imgvBG;
@property (retain, nonatomic) IBOutlet UIImageView *imgvBG2;
@property (retain, nonatomic) IBOutlet UIImageView *imgvLogo;
@property (retain, nonatomic) IBOutlet UIImageView *imgv20thLogo;

@property (retain, nonatomic) IBOutletCollection(UIButton) NSArray *openHouseButtons;


@end

@implementation MenuViewController

-(void)viewDidLoad
{
    [super viewDidLoad];
    
    [self.navigationController setNavigationBarHidden:YES];

    [self configureButtons];
    
    [[UITableViewCell appearance] setBackgroundColor:[UIColor clearColor]];

#ifdef USE_GA
    id<GAITracker> tracker = [[GAI sharedInstance] defaultTracker];
    [tracker set:kGAIScreenName value:@"Main Menu Screen"];
    [tracker send:[[GAIDictionaryBuilder createScreenView] build]];
#endif

    self.view.backgroundColor = [UIColor blackColor];
}

-(void)configureButtons
{
    NSString* newGameTitle;
    NSString* continueTitle;
    NSString* loadTitle;
    NSString* saveTitle;
    NSString* settingsTitle;
    NSString* quitTitle;
    NSString* extrasTitle;
    NSString* tutorialTitle;
    if (self.nativeUIInterface->hasFontwithLowercase()){
        newGameTitle = [NSString stringWithCPPWString:Strings::getString(StringId::new_game)];
        continueTitle = [NSString stringWithCPPWString:Strings::getString(StringId::continue_)];
        loadTitle = [NSString stringWithCPPWString:Strings::getString(StringId::load_game)];
        saveTitle = [NSString stringWithCPPWString:Strings::getString(StringId::save_game)];
        settingsTitle = [NSString stringWithCPPWString:Strings::getString(StringId::settings)];
        quitTitle = [NSString stringWithCPPWString:Strings::getString(StringId::quit)];
        extrasTitle = [NSString stringWithCPPWString:Strings::getString(StringId::extras)];
        tutorialTitle = [NSString stringWithCPPWString:Strings::getString(StringId::tutorial)];
    }
    else { // Forcing capital letters if needed

        newGameTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::new_game)] uppercaseString];
        continueTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::continue_)] uppercaseString];
        loadTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::load_game)] uppercaseString];
        saveTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::save_game)] uppercaseString];
        settingsTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::settings)] uppercaseString];
        quitTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::quit)] uppercaseString];
        extrasTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::extras)] uppercaseString];
        tutorialTitle = [[NSString stringWithCPPWString:Strings::getString(StringId::tutorial)] uppercaseString];
    }
    
    [self.btnNewGame setTitle:newGameTitle forState:UIControlStateNormal];
    [self.btnContinue setTitle:continueTitle forState:UIControlStateNormal];
    [self.btnLoad setTitle:loadTitle forState:UIControlStateNormal];
    [self.btnSave setTitle:saveTitle forState:UIControlStateNormal];
    [self.btnSettings setTitle:settingsTitle forState:UIControlStateNormal];
    [self.btnQuit setTitle:quitTitle forState:UIControlStateNormal];
    [self.btnTutorial setTitle:tutorialTitle forState:UIControlStateNormal];
    [self.btnExtras setTitle:extrasTitle forState:UIControlStateNormal];
    
    [self.btnNewGame configureWithTheme];
    [self.btnContinue configureWithTheme];
    [self.btnLoad configureWithTheme];
    [self.btnSave configureWithTheme];
    [self.btnSettings configureWithTheme];
    [self.btnQuit configureWithTheme];
    [self.btnExtras configureWithTheme];
    [self.btnTutorial configureWithTheme];
    [self.btnAd configureWithTheme];
}

-(void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    [self configureControls];

    if (GameInfo::getGameType() != GAME_TYPE_IHNM) {
        
        [self getAd];

        _currentAdGame = self.nativeUIInterface->getCurrentCrossPromotionGame();
        
        // _currentAdGame = GAME_TYPE_SIMON1; //testing
        [self displayCorrectAdForType:_currentAdGame];
    }
    
    if(self.nativeUIInterface->isOpenHouseEnabled())
    {
        // change bg
        self.imgvBG.image = [UIImage imageNamed:@"bg_openhouse"];
        
        // enable hot corners
        [self.openHouseButtons makeObjectsPerformSelector:@selector(setEnabled:) withObject:@YES];
    }
    else
    {
        [self.openHouseButtons setValue:[NSNumber numberWithBool:YES] forKey:@"hidden"];
        [self.openHouseButtons makeObjectsPerformSelector:@selector(setEnabled:) withObject:@NO];
    }
    
    if (self.showSplash) {
        [self prepareForOpeningAnimation];
    }
}

-(void)prepareForOpeningAnimation
{
    // hide buttons
    self.btnAd.alpha = 0;
    self.btnContinue.alpha = 0;
    self.btnNewGame.alpha = 0;
    self.btnSave.alpha = 0;
    self.btnLoad.alpha = 0;
    self.btnSettings.alpha = 0;
    self.btnTutorial.alpha = 0;
    self.btnExtras.alpha = 0;
    self.btnQuit.alpha = 0;
    self.btnAd.alpha = 0;
    self.imgvAd.alpha = 0;
    self.btnShare.alpha = 0;

    self.imgvBG.layer.opacity = 0;
    self.imgvLogo.layer.opacity = 0;
    
    CGFloat initialY = self.imgv20thLogo.layer.position.y;
    self.imgv20thLogo.layer.position =  CGPointMake(self.imgv20thLogo.layer.position.x, (initialY + CGRectGetHeight(self.imgv20thLogo.layer.frame)));
//    self.imgv20thLogo.layer.opacity = 0;
}

-(void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    if (self.showSplash) {
        if (CURRENT_GAME_TYPE == GAME_TYPE_IHNM)
        {
            [self BackgroundAnimation:0];
            [self LogoAnimation:0.8];
            [self ButtonsAnimation:1.5];
        }
        else if (CURRENT_GAME_TYPE == GAME_TYPE_FOTAQ)
        {
            [self BackgroundSlideUpAnimation];
            [self LogoAnimation:1.5];
            [self ButtonsAnimation:2.5];
            [self SlideUpBottomAnimation];
        }
        else if (CURRENT_GAME_TYPE == GAME_TYPE_SIMON1)
        {
            [self BackgroundAnimation:0];
            [self Background2Animation:1];
            [self Background3Animation:1];
            [self LogoAnimation:1.5];
            [self ButtonsAnimation:3];
         //   [self SlideUpBottomAnimation];
        }
        else // Default Animation
        {
            [self ButtonsAnimation:2.5];
        }
    }
}


-(void)BackgroundAnimation:(int) delay
{
    
    CABasicAnimation *fadeInBGAnimation = [CABasicAnimation animation];
    fadeInBGAnimation.keyPath = @"opacity";
    fadeInBGAnimation.fromValue = @0;
    fadeInBGAnimation.toValue = @1.0;
    fadeInBGAnimation.duration = 1.5;
    fadeInBGAnimation.beginTime = CACurrentMediaTime() + delay;
    fadeInBGAnimation.fillMode = kCAFillModeForwards;
    fadeInBGAnimation.additive = YES;
    fadeInBGAnimation.removedOnCompletion = NO;
    
    [self.imgvBG.layer addAnimation:fadeInBGAnimation forKey:@"fadeInBG"];
    
}

-(void)Background2Animation:(int) delay
{
    
    CABasicAnimation *fadeInBG2Animation = [CABasicAnimation animation];
    fadeInBG2Animation.keyPath = @"opacity";
    fadeInBG2Animation.fromValue = @0;
    fadeInBG2Animation.toValue = @1.0;
    fadeInBG2Animation.duration = 1.5;
    fadeInBG2Animation.beginTime = CACurrentMediaTime() + delay;
    fadeInBG2Animation.fillMode = kCAFillModeForwards;
    fadeInBG2Animation.additive = YES;
    fadeInBG2Animation.removedOnCompletion = NO;
    
    [self.imgvBG2.layer addAnimation:fadeInBG2Animation forKey:@"fadeInBG2"];
    
}

-(void)Background3Animation:(int) delay
{
    
    CABasicAnimation *fadeInBG3Animation = [CABasicAnimation animation];
    fadeInBG3Animation.keyPath = @"opacity";
    fadeInBG3Animation.fromValue = @0;
    fadeInBG3Animation.toValue = @1.0;
    fadeInBG3Animation.duration = 1.5;
    fadeInBG3Animation.beginTime = CACurrentMediaTime() + delay;
    fadeInBG3Animation.fillMode = kCAFillModeForwards;
    fadeInBG3Animation.additive = YES;
    fadeInBG3Animation.removedOnCompletion = NO;
    
    [self.imgv20thLogo.layer addAnimation:fadeInBG3Animation forKey:@"fadeInBG2"];
    
}

-(void)BackgroundSlideUpAnimation
{
    
    // animate the bg from bottom
    CAKeyframeAnimation *bgSlideUpAnimation = [CAKeyframeAnimation animation];
    bgSlideUpAnimation.keyPath = @"position.y";
    bgSlideUpAnimation.values = @[ @(CGRectGetHeight(self.imgvBG.frame)), @10, @-10, @10, @0 ];
    bgSlideUpAnimation.keyTimes = @[ @0, @(0.5), @(0.8), @(0.9), @1 ];
    bgSlideUpAnimation.duration = 1;
    bgSlideUpAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    bgSlideUpAnimation.additive = YES;
    
    [self.imgvBG.layer addAnimation:bgSlideUpAnimation forKey:@"slideUp"];
    
    CABasicAnimation *fadeInBGAnimation = [CABasicAnimation animation];
    fadeInBGAnimation.keyPath = NSStringFromSelector(@selector(opacity));
    fadeInBGAnimation.fromValue = @0;
    fadeInBGAnimation.toValue = @1;
    fadeInBGAnimation.duration = 1;
    fadeInBGAnimation.additive = YES;
    
    [self.imgvBG.layer addAnimation:fadeInBGAnimation forKey:@"fadeIn"];
    self.imgvBG.layer.opacity = 1.0;

}

-(void)LogoAnimation:(int) delay
{

    CABasicAnimation *fadeInLogoAnimation = [CABasicAnimation animation];
    fadeInLogoAnimation.keyPath = @"opacity";
    fadeInLogoAnimation.fromValue = @0;
    fadeInLogoAnimation.toValue = @1.0;
    fadeInLogoAnimation.duration = 1.5;
    fadeInLogoAnimation.beginTime = CACurrentMediaTime() + delay;
    fadeInLogoAnimation.fillMode = kCAFillModeForwards;
    fadeInLogoAnimation.additive = YES;
    fadeInLogoAnimation.removedOnCompletion = NO;

    [self.imgvLogo.layer addAnimation:fadeInLogoAnimation forKey:@"fadeInLogo"];
    
}

-(void)ButtonsAnimation:(int) delay
{
    
    // fade in the buttons
    [UIView animateWithDuration:0.5  delay:delay options:0 animations:^{
        self.btnAd.alpha = 1;
        self.btnContinue.alpha = 1;
        self.btnNewGame.alpha = 1;
        self.btnSave.alpha = 1;
        self.btnLoad.alpha = 1;
        self.btnSettings.alpha = 1;
        self.btnTutorial.alpha = 1;
        self.btnExtras.alpha = 1;
        self.btnQuit.alpha = 1;
        self.btnAd.alpha = 1;
        self.imgvAd.alpha = 1;
        self.btnShare.alpha = 1;
        //        self.imgvLogo.alpha = 1;
        //        self.imgvBG.alpha = 1;
        //        self.imgv20thLogo.alpha = 1;
    } completion:^(BOOL finished) {
        self.showSplash = NO;
        
    }];
    
}

-(void)SlideUpBottomAnimation
{
    
    CGFloat initialY = self.imgv20thLogo.layer.position.y;
    CABasicAnimation *bgSlideUpBottomLogoAnimation = [CABasicAnimation animation];
    bgSlideUpBottomLogoAnimation.keyPath = @"position.y";
    bgSlideUpBottomLogoAnimation.fromValue = @(initialY + CGRectGetHeight(self.imgv20thLogo.layer.frame));
    bgSlideUpBottomLogoAnimation.toValue = @(initialY);
    bgSlideUpBottomLogoAnimation.duration = 1;
    bgSlideUpBottomLogoAnimation.beginTime = CACurrentMediaTime() + 2.5;
    bgSlideUpBottomLogoAnimation.fillMode = kCAFillModeBackwards;
    bgSlideUpBottomLogoAnimation.removedOnCompletion = NO;
    
    bgSlideUpBottomLogoAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    
    [self.imgv20thLogo.layer addAnimation:bgSlideUpBottomLogoAnimation forKey:@"position"];
    
}

-(void)configureControls
{
    self.btnContinue.enabled = self.nativeUIInterface->isContinueEnabled()?YES:NO;
    self.btnSave.enabled = self.nativeUIInterface->isSaveEnabled()?YES:NO;
    self.btnLoad.enabled = self.nativeUIInterface->isLoadEnabled()?YES:NO;
}

- (IBAction)newGameTapped:(UIButton *)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onNewGamePressed();
}
- (IBAction)saveTapped:(id)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onSavePressed();
}
- (IBAction)loadTapped:(id)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onLoadPressed();
}
- (IBAction)continueTapped:(id)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onContinuePressed();
}
- (IBAction)settingsTapped:(id)sender {
    [buttonClickPlayer() play];

    self.nativeUIInterface->onSettingsFromMainMenu();
    
    [self.navigationController pushViewController:[SettingsViewController settingsViewController:self.nativeUIInterface] animated:NO];
}
- (IBAction)tutorialTapped:(id)sender
{
    [buttonClickPlayer() play];

    // show tutorial video
    self.nativeUIInterface->showTutorialVideo();
    
}
- (IBAction)extrasTapped:(id)sender {
    
    vector<ExtraItem> extraItems = self.nativeUIInterface->getExtras();
    
    ExtraItemsListViewController* extrasList = [ExtraItemsListViewController extrasListWithItems:extraItems];
    
    [self.navigationController pushViewController:extrasList animated:NO];
}

- (IBAction)adTapped:(id)sender {
    [buttonClickPlayer() play];

    if ([self.btnAd titleForState:UIControlStateNormal] != nil) {
        self.nativeUIInterface->onQuitPressed();
    }
    else
    {
        [self openStorePageForAd];
    }
}

-(void)openStorePageForAd
{
    MTStoreProductViewController *storeController = [[MTStoreProductViewController alloc] init];
    storeController.delegate = self;
    NSDictionary *productParameters = @{ SKStoreProductParameterITunesItemIdentifier : AppStoreIDForGameType(_currentAdGame) };
    [storeController loadProductWithParameters:productParameters completionBlock:^(BOOL result, NSError *error) {
        //Handle response
    }];
    
    [self presentViewController:storeController animated:YES completion:nil];
}

- (IBAction)quitTapped:(id)sender
{
    self.nativeUIInterface->onQuitPressed();
}

- (IBAction)hotCornerTapped:(id)sender {
    self.nativeUIInterface->onOpenHousePressed();
}


-(IBAction)share:(id)sender
{
    UIActivityViewController* activityViewController = [UIActivityViewController shareMyAppActivityViewController];
    
    BOOL isIpad = [UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad;

    
    if (isIpad) {
        UIPopoverController* popoverController = [[UIPopoverController alloc] initWithContentViewController:activityViewController];
        
        [popoverController presentPopoverFromRect:[sender frame] inView:self.view permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
    }
    else
    {
        [self presentViewController:activityViewController animated:YES completion:nil];
    }
}

-(void)getAd
{
    NSLog(@"IOS:: cross promotion check URL: %s", self.nativeUIInterface->getCrossPromotionCheckUrl().c_str());
    
    NSURL* url = [NSURL URLWithString:[NSString stringWithCPPString:self.nativeUIInterface->getCrossPromotionCheckUrl()]];
    
    NSURLSessionDataTask* urlSessionDataTask = [[NSURLSession sharedSession] dataTaskWithURL:url completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
        
        NSString* string = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
        
        self.nativeUIInterface->onCrossPromotionHttpResponse([string cppString]);
        
    }];
    
    [urlSessionDataTask resume];
}

-(void)displayCorrectAdForType:(int)type
{
    UIImage* image = nil;
    NSString* title = nil;
    self.btnQuit.hidden = NO;
    
    switch (type) {
        case GAME_TYPE_FOTAQ:
            image = [UIImage imageNamed:@"ad_fotaq"];
            break;
        case GAME_TYPE_SIMON1:
            image = [UIImage imageNamed:@"ad_simon1"];
            break;
        case GAME_TYPE_SIMON2:
            image = [UIImage imageNamed:@"ad_simon2"];
            break;
        case GAME_TYPE_T7G:
            image = [UIImage imageNamed:@"ad_t7g"];
            break;
        case GAME_TYPE_ITE:
            image = [UIImage imageNamed:@"ad_ite"];
            break;
        case GAME_TYPE_TONYT:
            image = [UIImage imageNamed:@"ad_tonyt"];
            break;
        case GAME_TYPE_UNKNOWN:
            title = [self.btnQuit titleForState:UIControlStateNormal];
            self.btnQuit.hidden = YES;
            break;
        default:
            break;
    }
    
    [self.imgvAd setImage:image];
    [self.btnAd setTitle:title forState:UIControlStateNormal];
}

#pragma SKStoreKit product view delegate

-(void)productViewControllerDidFinish:(SKStoreProductViewController *)viewController
{
    [self dismissViewControllerAnimated:YES completion:nil];
}

@end
