//
//  PlatformInterface.cpp
//  scummvm
//
//  Created by Omer Gilad on 7/29/15.
//
//

#include "PlatformInterface.h"
#include "GameOverlayManager.h"
#include "GameStateManager.h"
#include "GameSettingsManager.h"


#include "ui/NativeUiInterfaceStub.h"


#include "common/system.h"
#include "engines/engine.h"


namespace Enhanced {

    PlatformInterface* PlatformInterface::instance()
    {
        if (g_system == NULL)
        {
            return NULL;
        }
        
        return g_system->getExtendedPlatformInterface();
    }
    
    PlatformInterface::~PlatformInterface() {}

    void PlatformInterface::onSurfaceChanged(uint32 width, uint32 height)
    {
        GameOverlayManager::instance()->setDisplayDimensions(width, height);
    }
    
    NativeUiInterface* PlatformInterface::getNativeUiInterface()
    {
        // Default implementation
        static NativeUiInterfaceStub stub;
        return &stub;
    }

    float PlatformInterface::adjustDrawTransformationX(uint16 x, uint16 width)
    {
        float fract = (float)x / (float)width;
        float adjusted = GameOverlayManager::instance()->adjustToDrawTransformationX(fract);
        
        return adjusted * width;
    }
    
    float PlatformInterface::adjustDrawTransformationY(uint16 y, uint16 height)
    {
        float fract = (float)y / (float)height;
        float adjusted = GameOverlayManager::instance()->adjustToDrawTransformationY(fract);
        
        return adjusted * height;
    }
    
    void PlatformInterface::beforeDrawGameTextureToScreen(Graphics::Surface* gameSurface)
    {
        if (g_engine == NULL)
            return;
        
        // Update game state variables, based on the game surface
        GameStateManager::instance()->updateGameState(gameSurface);
        
        // Setup needed overlay animations for the current frame
        GameOverlayManager::instance()->setupGameOverlays();
    }
    
    Graphics::Surface* PlatformInterface::getModifiedGamePixels()
    {
        return GameStateManager::instance()->getModifiedGamePixels();
    }

    
    void PlatformInterface::afterDrawGameTextureToScreen()
    {
        if (g_engine == NULL)
            return;
        
        GameOverlayManager::instance()->drawAllGameOverlays();
    }
    
    bool PlatformInterface::shouldDrawGameScreen()
    {
        return !(GameInfo::isSimonGame() && GameStateManager::instance()->isInAutoloadState());
    }
    
    void PlatformInterface::showMouse(bool show)
    {
        GameStateManager::instance()->onShowMouse(show);
    }
    
    void PlatformInterface::drawOverlayBitmap(OverlayBitmap* bitmap, float x, float y,
                                   float width, float height, float alpha, OverlayShaderType shaderType)
    {
        // Empty
    }

    void PlatformInterface::allocateBitmapInTexture(OverlayBitmap* bitmap)
    {
        // Empty
    }

    bool PlatformInterface::onTapEvent(int x, int y, bool doubleTap)
    {
        return Enhanced::GameOverlayManager::instance()->onTapEvent(x, y, doubleTap);
    }
    
    bool PlatformInterface::onScrollEvent(int x, int y)
    {
        return Enhanced::GameOverlayManager::instance()->onScrollEvent(x, y);
    }
    
    bool PlatformInterface::onFlingEvent(int x1, int y1, int x2, int y2)
    {
        return Enhanced::GameOverlayManager::instance()->onFlingEvent(x1, y1, x2, y2);
    }
    
    bool PlatformInterface::onDownEvent(int x, int y)
    {
        return Enhanced::GameOverlayManager::instance()->onDownEvent(x, y);
    }
    
    bool PlatformInterface::onUpEvent(int x, int y)
    {
        return Enhanced::GameOverlayManager::instance()->onUpEvent(x, y);
    }
    
    bool PlatformInterface::IsMT32SupportedDevice()
    {
        // Default implementation - return false
        return false;
    }
    
    Common::String PlatformInterface::getDefaultGraphicSetting()
    {
        // Default implementation - return original
        return SETTING_VALUE_GRAPHIC_MODE_ORIGINAL;
    }

    Common::String PlatformInterface::getGameFilePath()
    {
        return Common::String(".");
    }


}